module Views.DataTableView exposing (view, update, subscriptions, init, icon)

import Html exposing (Html, div, button, i, input, text, table, thead, tbody, tr, th, td)
import Html.Attributes
import Html.Events exposing (onClick, onInput)
import Model exposing (Global, View(..))
import Msg exposing (Msg(..))
import Data.DataLoader exposing (Entry)
import Config.DataTableConfig as Local exposing (Model)
import Utils exposing (codeToName)

icon : Html Msg
icon =
    i [ Html.Attributes.class "fa-solid fa-table fa-xl" ] []

propagate : Local.Msg -> Msg
propagate msg =
    GotDataTable msg

init : Model
init = { query = Nothing, offset = 0 }

update : Local.Msg -> Global -> Model -> ( Global, Model, Cmd Msg )
update msg global model = 
    case msg of
        Local.QueryChange query ->
            ( global, { model | query = Just query }, Cmd.none )
        
        Local.OffsetChange offset ->
            ( global, { model | offset = offset }, Cmd.none )


dataRow : Entry -> Html Msg
dataRow entry =
    tr [ Html.Attributes.class "table-row" ]
        [ td [ Html.Attributes.class "table-cell text-start" ] [ codeToName entry.state |> text ]
        , td [ Html.Attributes.class "table-cell text-start" ] [ codeToName entry.consulateCountry |> text ]
        , td [ Html.Attributes.class "table-cell text-start" ] [ text entry.consulate ]
        , td [ Html.Attributes.class "table-cell text-end" ] [ entry.year |> String.fromInt |> text ]
        , td [ Html.Attributes.class "table-cell text-end" ] [ entry.totalVisasApplied |> Maybe.map String.fromFloat |> Maybe.withDefault "N.A." |> text ]
        , td [ Html.Attributes.class "table-cell text-end" ] [ entry.totalVisasIssued |> Maybe.map String.fromFloat |> Maybe.withDefault "N.A." |> text ]
        ]

dataHeader : Html Msg
dataHeader =
    tr [ Html.Attributes.class "table-header-row" ]
        [ th [ Html.Attributes.class "table-cell text-start" ] [ text "State" ]
        , th [ Html.Attributes.class "table-cell text-start" ] [ text "Consulate Country" ]
        , th [ Html.Attributes.class "table-cell text-start" ] [ text "Consulate" ]
        , th [ Html.Attributes.class "table-cell text-end" ] [ text "Year" ]
        , th [ Html.Attributes.class "table-cell text-end" ] [ text "Total Visas (applied for)" ]
        , th [ Html.Attributes.class "table-cell text-end" ] [ text "Total Visas (issued)" ]
        ]

dataTable : List Entry -> Html Msg
dataTable entries =
    table [ Html.Attributes.class "table" ]
        [ thead [ Html.Attributes.class "table-header" ] [ dataHeader ]
        , tbody [ Html.Attributes.class "table-body" ] (List.map dataRow entries)
        ]

view : Global -> Model -> Html Msg
view global model =
    let
        items : Int
        items = 15

        totalList : List Entry
        totalList =
            let
                iquery : Maybe String
                iquery = model.query |> Maybe.andThen (String.toLower >> Just)
            in
            global.data
            |> List.filter (
                \item -> case iquery of
                    Nothing -> True
                    Just query ->
                        String.contains query (codeToName item.state |> String.toLower)
                        || String.contains query (codeToName item.consulateCountry |> String.toLower)
                        || String.contains query (String.toLower item.consulate)
            )

        subsetList : List Entry
        subsetList =
            totalList
            |> List.drop model.offset
            |> List.take items

        maxOffset : Int
        maxOffset =
            List.length totalList - items

        prevOffset : Int
        prevOffset =
            model.offset - items |> max 0

        nextOffset : Int
        nextOffset =
            model.offset + items |> min maxOffset

        handleOffsetChange : String -> Msg
        handleOffsetChange msg =
            String.toInt msg
            |> Maybe.withDefault model.offset
            |> Local.OffsetChange
            |> propagate

        handleQueryChange : String -> Msg
        handleQueryChange msg =
            msg |> Local.QueryChange |> propagate
    in
    div [ Html.Attributes.class "datatable-wrapper" ]
        [ div [ Html.Attributes.class "row-spaced" ]
            [ button [ Local.OffsetChange prevOffset |> propagate |> onClick ]
                [ i [ Html.Attributes.class "fa-solid fa-caret-left" ] [] ]
            , div [ Html.Attributes.class "text-center" ]
                [ text "Showing "
                , model.offset |> String.fromInt |> text
                , text " to "
                , model.offset - 1 + List.length subsetList |> String.fromInt |> text
                , text " of a total of "
                , List.length totalList |> String.fromInt |> text
                , text " entries."
                ]
            , button [ Local.OffsetChange nextOffset |> propagate |> onClick ]
                [ i [ Html.Attributes.class "fa-solid fa-caret-right" ] [] ]
            ]
        , div [ Html.Attributes.class "datatable-content" ]
            [ dataTable subsetList ]
        , div [ Html.Attributes.class "datatable-footer text-center" ]
            [ div []
                [ text "Search: "
                , input
                    [ Html.Attributes.type_ "text"
                    , Html.Attributes.placeholder "Enter..."
                    , model.query |> Maybe.withDefault "" |> Html.Attributes.value
                    , onInput handleQueryChange
                    ] []
                ]
            , div []
                [ text "Jump to Offset: "
                , input
                    [ Html.Attributes.type_ "number"
                    , Html.Attributes.placeholder "Enter..."
                    , Html.Attributes.min "0"
                    , String.fromInt maxOffset |> Html.Attributes.max 
                    , String.fromInt model.offset |> Html.Attributes.value 
                    , onInput handleOffsetChange
                    ] []
                ]
            ]
        ]

subscriptions : Global -> Model -> Sub Msg
subscriptions global model = 
    Sub.none